==> spread-5.4.1.16-py3-none-any.whl: spread_core/bam/engineries/shade.py <==
from spread_core import bam
from spread_core.mqtt.spread.topic import State
from spread_core.mqtt.spread.variable import Variable
from .enginery import Enginery, _EngineryFunit


class Shade(Enginery):
    def on_state(self, topic: State, variable: Variable):
        if topic.entity_address.id == self.recipe.id:
            self.publish_state(PositionLevel, None if variable.invalid else variable.value)
            if not variable.preliminary:
                self.publish_state(Motion, None if variable.invalid else Motion.STOPPED)
        else:
            super(Shade, self).on_state(topic, variable)


class Shutter(Shade):
    pass


class Curtain(Shade):
    pass


class Blind(Shade):
    POSITION_TYPE = 'Closeness'
    ANGLE_TYPE = 'Angle'

    angle = None

    def set_ingredient(self, ingredient_type, recipe: bam.Recipe):
        if ingredient_type == self.POSITION_TYPE:
            self._recipe = recipe
        elif ingredient_type == self.ANGLE_TYPE:
            self.angle = recipe
        else:
            raise ValueError('unknown ingredient type: %s' % ingredient_type)

    def recipe_for_funit(self, funit):
        if funit == RotateAngle:
            return PositionLevel, self.angle
        else:
            return super(Blind, self).recipe_for_funit(funit)

    def on_state(self, topic: State, variable: Variable):
        if topic.entity_address.id == self.angle.id:
            self.publish_state(RotateAngle, None if variable.invalid else RotateAngle.level2angle(variable.value))
            if not variable.preliminary:
                self.publish_state(Rotation, None if variable.invalid else Rotation.STOPPED)
        else:
            super(Blind, self).on_state(topic, variable)


class _ShadeFunit(_EngineryFunit):
    entity_type = Shade


Shade.funit_class = _ShadeFunit


"""
[от Vladimir Bulatov]
Motion (Set: Open/Close/Stop State (?): Opening/Closing/Stopped)

PositionLevel ( Set/State: [0..100]%)

Rotation (Set: Minus/Plus/Stop State (?): Minising/Plusing/Stopped)
Или вообще использовать cw/ccw

PositionAngle ( Set/State: [-90..90]градус)
"""


class Motion(_ShadeFunit):
    OPENING = 'Opening'
    CLOSING = 'Closing'
    STOPPED = 'Stopped'

    OPEN = 'Open'
    CLOSE = 'Close'
    STOP = 'Stop'

    variants = [OPEN, CLOSE, STOP]
    state_variants = [OPENING, CLOSING, STOPPED]

    def set(self):
        level = self.motion2level(self.set_value)

        if level is None:
            assert self.entity.get_value(self.__class__) != self.STOPPED, "Already stopped"

        PositionLevel(self.entity, level).set()

    @classmethod
    def motion2level(cls, motion: str) -> [None, int]:
        if motion == cls.CLOSE:
            return PositionLevel.value_range.stop - 1
        elif motion == cls.OPEN:
            return PositionLevel.value_range.start
        elif motion == cls.STOP:
            return None

        raise ValueError(f'"direction" expected as one of [{cls.OPEN}, {cls.CLOSE}, {cls.STOP}] but {motion} is given')


class PositionLevel(_ShadeFunit):
    value_range = range(0, 101)

    def set(self):
        has_level = self.entity.get_value(PositionLevel)
        super(PositionLevel, self).set()

        if has_level is not None:
            if self.set_value is None:
                pass
            elif self.set_value > has_level:
                self.entity.publish_state(Motion, Motion.CLOSING)
            elif self.set_value < has_level:
                self.entity.publish_state(Motion, Motion.OPENING)


class Rotation(_ShadeFunit):
    entity_type = Shutter

    MINUS = 'Minus'
    PLUS = 'Plus'
    STOP = 'Stop'

    MINISING = 'Minising'
    PLUSING = 'Plusing'
    STOPPED = 'Stopped'

    variants = [MINUS, PLUS, STOP]
    state_variants = [MINISING, PLUSING, STOPPED]

    def set(self):
        angle = self.direction2angle(self.set_value)

        if angle is None:
            assert self.entity.get_value(self.__class__) != self.STOPPED, "Already stopped"

        RotateAngle(self.entity, angle).set()

    @classmethod
    def direction2angle(cls, direct: str) -> [None, int]:
        if direct == cls.MINUS:
            return RotateAngle.value_range.start
        elif direct == cls.PLUS:
            return RotateAngle.value_range.stop-1
        elif direct == cls.STOP:
            return None

        raise ValueError(f'"direction" expected as one of [{cls.MINUS}, {cls.PLUS}, {cls.STOP}] but {direct} is given')


class RotateAngle(_ShadeFunit):
    entity_type = Shutter
    value_range = range(-90, 91)

    def set(self):
        has_angle = self.entity.get_value(RotateAngle)
        super(RotateAngle, self).set()

        if has_angle is not None:
            if self.set_value is None:
                pass
            elif self.set_value > has_angle:
                self.entity.publish_state(Rotation, Rotation.PLUSING)
            elif self.set_value < has_angle:
                self.entity.publish_state(Rotation, Rotation.MINISING)

    @property
    def set_value(self):
        return self.angle2level(super(RotateAngle, self).set_value)

    @staticmethod
    def level2angle(raw):
        return round(180 * raw / 100) - 90

    @staticmethod
    def angle2level(angle):
        if angle is not None:
            return round(100 * (angle + 90) / 180)

        return angle


Shade.settable_units = [
    Motion,
    PositionLevel,
]

Blind.settable_units = Shade.settable_units + [
    Rotation,
    RotateAngle
]

==> spread-5.5.0.34-py3-none-any.whl: spread_core/bam/engineries/shade.py <==
from spread_core.mqtt.spread.topic import State
from spread_core.mqtt.spread.variable import Variable
from .enginery import Enginery, _EngineryFunit
from ..recipe import Recipe


class Shade(Enginery):
    def on_state(self, topic: State, variable: Variable):
        if topic.entity_address.id == self._recipe.id:
            self.publish_state(PositionLevel, None if variable.invalid else variable.value)
            if not variable.preliminary:
                self.publish_state(Motion, None if variable.invalid else Motion.STOPPED)
        else:
            super(Shade, self).on_state(topic, variable)


class Shutter(Shade):
    pass


class Curtain(Shade):
    pass


class Blind(Shade):
    POSITION_TYPE = 'Closeness'
    ANGLE_TYPE = 'Angle'

    def __init__(self, _id: int, **kwargs):
        super(Blind, self).__init__(_id, **kwargs)
        self._angle = None

    def set_recipe(self, ingredient_type, recipe: Recipe):
        if ingredient_type == self.POSITION_TYPE:
            self._recipe = recipe
        elif ingredient_type == self.ANGLE_TYPE:
            self._angle = recipe
        else:
            raise ValueError('unknown ingredient type: "%s"' % ingredient_type)

    def recipe_for_funit(self, funit):
        if funit == RotateAngle:
            return PositionLevel, self._angle
        else:
            return super(Blind, self).recipe_for_funit(funit)

    def on_state(self, topic: State, variable: Variable):
        if topic.entity_address.id == self._angle.id:
            self.publish_state(RotateAngle, None if variable.invalid else RotateAngle.level2angle(variable.value))
            if not variable.preliminary:
                self.publish_state(Rotation, None if variable.invalid else Rotation.STOPPED)
        else:
            super(Blind, self).on_state(topic, variable)


class _ShadeFunit(_EngineryFunit):
    entity_type = Shade


Shade.funit_class = _ShadeFunit


"""
[от Vladimir Bulatov]
Motion (Set: Open/Close/Stop State (?): Opening/Closing/Stopped)

PositionLevel ( Set/State: [0..100]%)

Rotation (Set: Minus/Plus/Stop State (?): Minising/Plusing/Stopped)
Или вообще использовать cw/ccw

PositionAngle ( Set/State: [-90..90]градус)
"""


class Motion(_ShadeFunit):
    OPENING = 'Opening'
    CLOSING = 'Closing'
    STOPPED = 'Stopped'

    OPEN = 'Open'
    CLOSE = 'Close'
    STOP = 'Stop'

    variants = [OPEN, CLOSE, STOP]
    state_variants = [OPENING, CLOSING, STOPPED]

    def set(self):
        level = self.motion2level(self.set_value)

        if level is None:
            assert self.entity.get_value(self.__class__) != self.STOPPED, "Already stopped"

        PositionLevel(self.entity, level).set()

    @classmethod
    def motion2level(cls, motion: str) -> [None, int]:
        if motion == cls.CLOSE:
            return PositionLevel.value_range.stop - 1
        elif motion == cls.OPEN:
            return PositionLevel.value_range.start
        elif motion == cls.STOP:
            return None

        raise ValueError(f'"direction" expected as one of [{cls.OPEN}, {cls.CLOSE}, {cls.STOP}] but {motion} is given')


class PositionLevel(_ShadeFunit):
    value_range = range(0, 101)

    def set(self):
        has_level = self.entity.get_value(PositionLevel)
        super(PositionLevel, self).set()

        if has_level is not None:
            if self.set_value is None:
                pass
            elif self.set_value > has_level:
                self.entity.publish_state(Motion, Motion.CLOSING)
            elif self.set_value < has_level:
                self.entity.publish_state(Motion, Motion.OPENING)


class Rotation(_ShadeFunit):
    entity_type = Shutter

    MINUS = 'Minus'
    PLUS = 'Plus'
    STOP = 'Stop'

    MINISING = 'Minising'
    PLUSING = 'Plusing'
    STOPPED = 'Stopped'

    variants = [MINUS, PLUS, STOP]
    state_variants = [MINISING, PLUSING, STOPPED]

    def set(self):
        angle = self.direction2angle(self.set_value)

        if angle is None:
            assert self.entity.get_value(self.__class__) != self.STOPPED, "Already stopped"

        RotateAngle(self.entity, angle).set()

    @classmethod
    def direction2angle(cls, direct: str) -> [None, int]:
        if direct == cls.MINUS:
            return RotateAngle.value_range.start
        elif direct == cls.PLUS:
            return RotateAngle.value_range.stop-1
        elif direct == cls.STOP:
            return None

        raise ValueError(f'"direction" expected as one of [{cls.MINUS}, {cls.PLUS}, {cls.STOP}] but {direct} is given')


class RotateAngle(_ShadeFunit):
    entity_type = Shutter
    value_range = range(-90, 91)

    def set(self):
        has_angle = self.entity.get_value(RotateAngle)
        super(RotateAngle, self).set()

        if has_angle is not None:
            if self.set_value is None:
                pass
            elif self.set_value > has_angle:
                self.entity.publish_state(Rotation, Rotation.PLUSING)
            elif self.set_value < has_angle:
                self.entity.publish_state(Rotation, Rotation.MINISING)

    @property
    def set_value(self):
        return self.angle2level(super(RotateAngle, self).set_value)

    @staticmethod
    def level2angle(raw):
        return round(180 * raw / 100) - 90

    @staticmethod
    def angle2level(angle):
        if angle is not None:
            return round(100 * (angle + 90) / 180)

        return angle


Shade.settable_units = [
    Motion,
    PositionLevel,
]

Blind.settable_units = Shade.settable_units + [
    Rotation,
    RotateAngle
]
